unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, _Base, Menus, StdCtrls, ComCtrls, Gauges, ExtCtrls, ToolWin;

type
  TMainForm = class(T_BaseForm)
    MainMenu: TMainMenu;
    MenuFile: TMenuItem;
    MenuMisc: TMenuItem;
    MenuAbout: TMenuItem;
    MenuDownLoad: TMenuItem;
    N1: TMenuItem;
    MenuExit: TMenuItem;
    Memo1: TMemo;
    StatusBar: TStatusBar;
    MenuDebug: TMenuItem;
    MenuLoad: TMenuItem;
    CoolBar1: TCoolBar;
    ComboKeyWord: TComboBox;
    N2: TMenuItem;
    MenuIgnoreCase: TMenuItem;
    procedure MenuAboutClick(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure MenuDownLoadClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MenuDebugClick(Sender: TObject);
    procedure MenuLoadClick(Sender: TObject);
    procedure ComboKeyWordKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Memo1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MenuIgnoreCaseClick(Sender: TObject);
  private
    Finder      : TStringList;
    Stream      : TMemoryStream;
    procedure   WriteMessage(AMsg: string);
    procedure   EdictScan(AKeyWord: string; AResult: TStrings);
    function    AddKeyWord(AKeyWord: string): Boolean;

    procedure   LoadFormInfo();
    procedure   SaveFormInfo();
  public

  end;

var
  MainForm: TMainForm;

implementation

uses
    StrUtils, Wininet, IniFiles, LoadFile, Info, iFiles, jeuc, memdbg;

{$R *.dfm}

(*
    ============================================================================
    ϐ
    ============================================================================
*)
var
    USER_AGENT    : string = 'EdictScanner20030113';
    EDICT_FILENAME: string = 'edict';
    EDICT_URL     : string = 'http://ftp.cc.monash.edu.au/pub/nihongo/edict.zip';
    EDICT_ZIP     : string = 'edict.zip';
    LIMIT_COUNT   : integer = 200;
    IGNORE_CASE   : Boolean = true;

(*
    ============================================================================
    [eBeB
    ============================================================================
*)

(*
    ===========================================================================
    񃁃bZ[W_CAO
    ===========================================================================
*)
procedure   SimpleMessage(AMsg: string);
var
    Dialog: TForm;
    Image : TImage;
begin
    Application.NormalizeTopMosts;

    Dialog := CreateMessageDialog(AMsg, mtInformation, [mbOK]);
    try
        Image := TImage(Dialog.FindComponent('Image'));
        if (Image <> nil) then Image.Picture.Icon.Assign(Application.Icon);
        Dialog.ShowModal;
    finally
        Dialog.Free;
    end;

    Application.RestoreTopMosts;
end;

(*
    ===========================================================================
    mFbZ[W_CAO
    ===========================================================================
*)
function    SimpleYesNo(AMsg: string): Boolean;
var
    Dialog: TForm;
    Image : TImage;
begin
    Application.NormalizeTopMosts;

    Dialog := CreateMessageDialog(AMsg, mtInformation, [mbYes, mbNo]);
    try
        Image := TImage(Dialog.FindComponent('Image'));
        if (Image <> nil) then Image.Picture.Icon.Assign(Application.Icon);
        Result := (Dialog.ShowModal = mrYes);
    finally
        Dialog.Free;
    end;
    
    Application.RestoreTopMosts;
end;

(*
    ============================================================================
    ̑
    ============================================================================
*)
procedure   TMainForm.LoadFormInfo();
var
    IniFile: TIniFile;
begin
    IniFile := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
    try
        USER_AGENT := IniFile.ReadString('download', 'UserAgent', USER_AGENT);
        EDICT_URL := IniFile.ReadString('download', 'EdictUrl', EDICT_URL);
        EDICT_ZIP := IniFile.ReadString('download', 'EdictZip', EDICT_ZIP);
        EDICT_FILENAME := IniFile.ReadString('download', 'EdictFileName', EDICT_FILENAME);

        LIMIT_COUNT := IniFile.ReadInteger('Setting', 'LimitCount', LIMIT_COUNT);
        IGNORE_CASE := IniFile.ReadBool('Setting', 'IgnoreCase', IGNORE_CASE);

        Self.Left := IniFile.ReadInteger('Setting', 'FormLeft', Self.Left);
        Self.Top := IniFile.ReadInteger('Setting', 'FormTop', Self.Top);
        Self.Height := IniFile.ReadInteger('Setting', 'FormHeight', Self.Height);
        Self.Width := IniFile.ReadInteger('Setting', 'FormWidth', Self.Width);

        MenuIgnoreCase.Checked := IGNORE_CASE;
    finally
        IniFile.Free;
    end;
end;

procedure   TMainForm.SaveFormInfo();
var
    IniFile: TIniFile;
begin
    IniFile := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
    try
        IniFile.WriteString('download', 'UserAgent', USER_AGENT);
        IniFile.WriteString('download', 'EdictUrl', EDICT_URL);
        IniFile.WriteString('download', 'EdictZip', EDICT_ZIP);
        IniFile.WriteString('download', 'EdictFileName', EDICT_FILENAME);

        IniFile.WriteInteger('Setting', 'LimitCount', LIMIT_COUNT);
        IniFile.WriteBool('Setting', 'IgnoreCase', IGNORE_CASE);

        IniFile.WriteInteger('Setting', 'FormLeft', Self.Left);
        IniFile.WriteInteger('Setting', 'FormTop', Self.Top);
        IniFile.WriteInteger('Setting', 'FormHeight', Self.Height);
        IniFile.WriteInteger('Setting', 'FormWidth', Self.Width);

        IniFile.UpdateFile();
    finally
        IniFile.Free;
    end;
end;

(*
    ============================================================================
    Cxgnh
    ============================================================================
*)

//  bZ[Wo
procedure   TMainForm.WriteMessage(AMsg: string);
begin
    StatusBar.SimpleText := AMsg;
end;

//  tH[RXgN^
procedure TMainForm.FormCreate(Sender: TObject);
begin
  inherited;
    Stream := TMemoryStream.Create;
    Finder := TStringList.Create;
    LoadFormInfo();
    MenuLoadClick(MenuLoad);
end;

//  tH[fXgN^
procedure TMainForm.FormDestroy(Sender: TObject);
begin
    SaveFormInfo();
    Stream.Free;
    Finder.Free;
  inherited;
end;

(*
    ============================================================================
    j[Cxgnh
    ============================================================================
*)

//  I
procedure TMainForm.MenuExitClick(Sender: TObject);
begin
    //  
    Self.Close();
end;

//  
procedure TMainForm.MenuAboutClick(Sender: TObject);
begin
    OpenInfoForm();
end;

//  fobO
procedure TMainForm.MenuDebugClick(Sender: TObject);
begin
    MemDebugForm.Show;
end;

//  啶/ǂ邩
procedure TMainForm.MenuIgnoreCaseClick(Sender: TObject);
begin
    TMenuItem(Sender).Checked := not TMenuItem(Sender).Checked;
    IGNORE_CASE := TMenuItem(Sender).Checked;
end;

//  _E[h
procedure TMainForm.MenuDownLoadClick(Sender: TObject);
const
    MSG = '_E[ĥŁA𓀂ĂB'#13#10
        + '̌vOƓtH_Ɏt@C'#13#10
        + 'ĂB̂Ŏt@Cǂ'#13#10
        + '݂܂̂ŁAȂꍇ͉łu_E[h'#13#10
        + '܂Hvĕ܂B';
var
    Res: Boolean;
begin
    WriteMessage('DownLoad from [' + EDICT_URL + ']');
    Res := LoadFromNetFile(EDICT_URL, USER_AGENT, Stream);
    WriteMessage('');
    if Res then
    begin
        Stream.SaveToFile(AppDirPath + EDICT_ZIP);
        SimpleMessage(MSG);
        SimpleShellExecute(AppDirPath);
    end;
end;

//  t@C[h
procedure TMainForm.MenuLoadClick(Sender: TObject);
const
    MSG = 't@CȂłˁB{Iɂ1s1R['#13#10
        + 'h̎΂悢̂łÃvO'#13#10
        + 'Edict Ƃt@Cg܂̂ŁA_E'#13#10
        + '[hĂ炢łǂł傤H'#13#10
        + 'ŏṼvOŃ_E['#13#10
        + 'VB_E[h܂H';
begin
    if FileExists(AppDirPath + EDICT_FILENAME) then
    begin
        WriteMessage('Load [' + EDICT_FILENAME + ']');
        LoadFromLocalFile(AppDirPath + EDICT_FILENAME, Finder);
        WriteMessage('');
    end
    else
    begin
        if SimpleYesNo(MSG) then
        begin
            MenuDownLoadClick(MenuDownLoad);
        end;
    end;
end;

//  L[[h𗚗
//  OƓȂfalse
function    TMainForm.AddKeyWord(AKeyWord: string): Boolean;
var
    Index: integer;
begin
    Result := true;
    //  1ȏ゠
    if (ComboKeyWord.Items.Count > 0) then
    begin
        //  ŐV̌Ɠǂ`FbN
        if (ComboKeyWord.Items[0] <> AKeyWord) then
        begin
            //  ႤƂĂߋ̂ǂłЂƂ猩邩...
            Index := ComboKeyWord.Items.IndexOf(AKeyWord);
            if (Index <> -1) then
            begin
                ComboKeyWord.Items.Move(Index, 0);
            end
            else
            begin
                ComboKeyWord.Items.Insert(0, AKeyWord);
            end;
        end
        else
        begin
            Result := false;
        end;
    end
    else
    //  ߂ĂȂ
    begin
        ComboKeyWord.Items.Insert(0, AKeyWord);
    end;
    //  ŁA0Ԗڂ\
    ComboKeyWord.Text := ComboKeyWord.Items[0]; 
end;

//  ܗL`FbN֐
type
    TCheckContainsText = function(Text, SubText: string): Boolean;

//  啶
function    CheckContainsText(Text, SubText: string): Boolean;
begin
    Result := AnsiContainsText(Text, SubText);
end;

//  啶l
function    CheckContainsStr(Text, SubText: string): Boolean;
begin
    Result := AnsiContainsStr(Text, SubText);
end;

//  ܗL`FbN֐ϐ(֐|C^)
var
    ContainsChecker: array[Boolean] of TCheckContainsText =(
        CheckContainsStr, CheckContainsText
    );

//  
procedure   TMainForm.EdictScan(AKeyWord: string; AResult: TStrings);
var
    i: integer;
    EucKeyword: string;
begin
    AResult.Clear;
    EucKeyword := SJIStoEUC_String(Trim(AKeyWord));
    if (EucKeyword = '') then Exit;

    for i := 0 to Finder.Count - 1 do
    begin
        if ContainsChecker[IGNORE_CASE](Finder[i], EucKeyword) then
        begin
            AResult.Add(Format('[%6.6d] %s', [i, EUCtoSJIS_String(Finder[i])]));
        end;
        if (AResult.Count >= LIMIT_COUNT) then Break;
    end;

    AResult.Add(StringOfChar('=', 100));
    AResult.Add(IntToStr(AResult.Count - 1{===̕}) + 'Records HIT!');
end;

//  
procedure TMainForm.ComboKeyWordKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
    KeyWord: string;
begin
    if (Key = VK_RETURN) then
    begin
        KeyWord := ComboKeyWord.Text;
        WriteMessage('Now Scanning for [' + KeyWord + ']');
        if AddKeyWord(KeyWord) then
        begin
            EdictScan(KeyWord, Memo1.Lines);
        end;
        WriteMessage('');
    end;
end;

(*
    *******************************************************
    TMemǒݍs̎擾
    *******************************************************
*)
function GetMemoCurrentLine(AMemo: TMemo): integer;
begin
    with AMemo do
        Result := SendMessage(Handle, EM_LINEFROMCHAR, SelStart, 0);
end;
(*
    *******************************************************
    TMemǒݍs̐擪̃oCg擾
    *******************************************************
*)
function GetMemoCrrentCol(AMemo: TMemo): integer;
begin
    with AMemo do
        Result := SelStart
                - SendMessage(Handle,
                              EM_LINEINDEX,
                              GetMemoCurrentLine(AMemo),
                              0);
end;

//  Iꌟ
procedure TMainForm.Memo1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
    KeyWord: string;
    Row, Col, TempCol: integer;
    CurrentLine: string;
    TempChar: Char;
begin
    if (Key = VK_RETURN) then
    begin
        //  IĂ܂H
        KeyWord := Trim(Memo1.SelText);
        //  Ă܂
        if (KeyWord = '') then
        begin

            try

            //  J[\ʒu擾
            Row := GetMemoCurrentLine(Memo1);
            CurrentLine := Memo1.Lines[Row];
            if (CurrentLine <> '') then
            begin
                Col := GetMemoCrrentCol(Memo1);
                //  J[\ʒȗOɂPۂ擾
                TempCol := Col;
                while true do
                begin
                    Dec(TempCol);
                    if (TempCol <= 0) then break;
                    TempChar := CurrentLine[TempCol];
                    if (TempChar in [' ', #9, #10, #13]) then break;
                    Insert(TempChar, KeyWord, 1);
                end;
                //  J[\ʒǔɂPۂ擾
                TempCol := Col;
                KeyWord := KeyWord + CurrentLine[TempCol];
                while true do
                begin
                    Inc(TempCol);
                    if (TempCol > Length(CurrentLine)) then break;
                    TempChar := CurrentLine[TempCol];
                    if (TempChar in [' ', #9, #10, #13]) then break;
                    KeyWord := KeyWord + CurrentLine[TempCol];
                end;
            end;

            except
                KeyWord := 'I''m sorry... bug';
            end;

        end;
        //  R{ɒǉ
        WriteMessage('Now Scanning for [' + KeyWord + ']');
        if AddKeyWord(KeyWord) then
        begin
            EdictScan(KeyWord, Memo1.Lines);
        end;
        WriteMessage('');
    end;
end;

end.

