unit iFiles;

interface

uses
    Windows, Classes, SysUtils, ExtCtrls, FileCtrl, ShellApi, Forms, Dialogs;

type
    TOnFindFile = procedure(const APath: string; const ALevel: integer; var Stop: Boolean) of object;
    TOnFindFolder = procedure(const APath: string; const ALevel: integer; var Skip: Boolean; var Stop: Boolean) of object;

function    FindFiles(const ADirPath: string; const ALevel: integer;
                ACallbackFile: TOnFindFile; ACallbackFolder: TOnFindFolder): Boolean;
function    SelectFolder(const ARoot: string = 'C:\'; ACaption: string = 'tH_̑I'): string;
function    GetTempFolder(): string;
procedure   LoadFromFile(APath: string; AStrings: TStrings; IgnoreZeroLine: Boolean = False);
function    ExecuteFile(FilePath: string; Params: string): Boolean;
function    CheckExtension(AFileName, AExtension: string): Boolean;
procedure   PutLog(AStr: string);   overload;
procedure   PutLog(AStr: integer);  overload;
procedure   PutLog(AStr: double);   overload;
procedure   PutLogEx(AStr: string); overload;
procedure   PutLogEx(AStr: integer);overload;
procedure   PutLogEx(AStr: double); overload;

procedure   SimpleShellExecute(const FileName: string);
procedure   SendToRecycleBin(FileName: string);                         overload;
procedure   SendToRecycleBin(AFiles: TStrings; ANeedConfirm: Boolean);  overload;
procedure   OpenFilePropertyDialog(AFilePath: string);

type
    TSpecialFolder = (
        sfDesktop, sfProgram, sfControls, sfPrinters, sfPersonal{, sfFovorites}
        , sfStarup, sfRecent, sfSendTo, sfBitBucket, sfStartMenu
        , sfDesktopDirectory, sfDrieve, sfNetWork, sfNetHood, sfFonts
        {, sfTemplete}, sfAppData, sfPrintHood
    );
const
    SpecialFolderStr: array[TSpecialFolder] of string =
    (
        'fXNgbv(ztH_)', 'vOj[', 'Rg[pl(ztH_)'
        , 'v^(ztH_)', '}ChLg'{, 'Cɓ'}, 'X^[gAbv'
        , 'ŋߎgt@C', '', 'S~', 'X^[gj[', 'fXNgbv'
        , '}CRs[^(ztH_)', 'lbg[N(ztH_)'
        , 'lbg[NRs[^', 'tHg'{, 'ev[g'}
        , 'AvP[Vf[^', 'v^'
    );
function    GetSpecialFolderPath(nFolder: TSpecialFolder):string;

var
    AppExePath: string;
    AppExeName: string;
    AppDirPath: string;
    AppLogPath: string;
    AppIniPath: string;
    AppDatPath: string;

implementation

uses
    ShlObj, ActiveX;

(*
    ===========================================================================
    etH_̃pX擾
    ---------------------------------------------------------------------------
    http://homepage1.nifty.com/MADIA/delphi/Win32API/ShFolder.htm
    http://homepage2.nifty.com/Mr_XRAY/Delphi/plFileDialog/B_BrowseFolder.htm
    ===========================================================================
*)
const
    SpecialFolderID: array[TSpecialFolder] of integer =
    (
        CSIDL_DESKTOP, CSIDL_PROGRAMS, CSIDL_CONTROLS, CSIDL_PRINTERS
        , CSIDL_PERSONAL{, CSIDL_FOVORITES}, CSIDL_STARTUP, CSIDL_RECENT
        , CSIDL_SENDTO, CSIDL_BITBUCKET, CSIDL_STARTMENU, CSIDL_DESKTOPDIRECTORY
        , CSIDL_DRIVES, CSIDL_NETWORK, CSIDL_NETHOOD, CSIDL_FONTS{, CSIDL_TEMPLATE}
        , CSIDL_APPDATA, CSIDL_PRINTHOOD
    );

function    GetSpecialFolderPath(nFolder: TSpecialFolder):string;
var
    Root : PItemIDList;
    Buffer: PChar;
    Memory: IMalloc;
begin
    SHGetMalloc(Memory);
    Buffer := Memory.Alloc(MAX_PATH);
    //  tH_̈ʒu擾
    SHGetSpecialFolderLocation(Application.MainForm.Handle, SpecialFolderID[nFolder], Root);
    //  ItemIDList\̂pXo
    SHGetPathFromIDList(Root, Buffer);
    Result := Buffer;
    //  ̉
    Memory.Free(Root);
    Memory.Free(Buffer);
end;

(*
    =====================================================================
    t@C̓ǂݍ
    =====================================================================
*)
procedure   LoadFromFile(APath: string; AStrings: TStrings; IgnoreZeroLine: Boolean = False);
var
    F: TextFile;
    S: string;
begin
    AStrings.Clear;
    AssignFile(F, APath);
    Reset(F);
    while not EOF(F) do begin
        Readln(F, S);
        if IgnoreZeroLine and (Length(Trim(S)) = 0) then Continue;
        AStrings.Add(S);
    end;
    CloseFile(F);
end;

(*
    =====================================================================
    ShellExecutẽbp[֐
    =====================================================================
*)
function    ExecuteFile(FilePath: string; Params: string): Boolean;
const
    ERROR_NO = 32;
var
    Value: integer;
begin
    Value := ShellExecute(
                    Application.MainForm.handle,
                    'open',
                    PChar(FilePath),
                    PChar(Params),
                    PChar(ExtractFilePath(Application.Exename)),
                    SW_SHOWDEFAULT
                    );
    Result := (Value > ERROR_NO);
end;

(*
    ===========================================================================
    ShellExecutebp[
    ===========================================================================
*)
procedure   SimpleShellExecute(const FileName: string);
begin
    try
        ShellExecute(Application.MainForm.Handle, nil, PChar(FileName), nil, nil, SW_SHOW);
    except
        MessageDlg('Hi!', mtInformation, [mbOk], 0);
    end;
end;

(*
    =====================================================================
    gq`FbN
    =====================================================================
*)
function    CheckExtension(AFileName, AExtension: string): Boolean;
begin
    Result := (UpperCase(ExtractFileExt(AFileName)) = UpperCase(AExtension));
end;

(*
    =====================================================================
    Oo
    =====================================================================
*)

// 1
procedure    PutLog(AStr: string);
var
    F: TextFile;
begin
    if AStr = '' then Exit;
    AssignFile(F, AppLogPath);
    try
        if FileExists(AppLogPath) then Append(F) else Rewrite(F);
        Writeln(f, AStr);
        Flush(F);
    finally
        CloseFile(F);
    end;
end;
// 2
procedure    PutLog(AStr: integer);
begin
    PutLog(IntToStr(AStr));
end;
// 3
procedure    PutLog(AStr: double);
begin
    PutLog(FloatToStr(AStr));
end;
// 4
procedure    PutLogEx(AStr: string);
begin
    if AStr = '' then Exit;
    PutLog(FormatDateTime('YYYY/MM/DD HH:NN:SS', Now) + ' >> ' + AStr);
end;
// 5
procedure    PutLogEx(AStr: integer);
begin
    PutLogEx(IntToStr(AStr));
end;
// 6
procedure    PutLogEx(AStr: double);
begin
    PutLogEx(FloatToStr(AStr));
end;

(*
    =====================================================================
    ݔs
    =====================================================================
*)

procedure    SendToRecycleBin(FileName: string);
var
    SHF: TSHFileOpStruct;
begin
    with SHF do
    begin
        Wnd    := Application.Handle;
        wFunc  := FO_DELETE;
        pFrom  := PChar(FileName + #0);
        fFlags := FOF_ALLOWUNDO;
    end;
    SHFileOperation(SHF);
end;

(*
    ===========================================================================
    t@C폜ăS~Ɉړ
    ---------------------------------------------------------------------------
    http://www2.big.or.jp/~osamu/Delphi/Tips/key.cgi?key=2#0275.txt
    ===========================================================================
*)
procedure   SendToRecycleBin(AFiles: TStrings; ANeedConfirm: Boolean);
const
    SHF_FLAG: array[Boolean] of WORD = (
        FOF_ALLOWUNDO or FOF_NOCONFIRMATION
       ,FOF_ALLOWUNDO
    );
var
    SHF: TSHFileOpStruct;
    FileNames: string;
    i: integer;
begin
    if AFiles.Count = 0 then exit;
    //  #0f~^Ƃ
    for i := 0 to AFiles.Count - 1 do
    begin
        FileNames := FileNames + AFiles[i] + #0;
    end;
    //  #0
    FileNames := FileNames + #0;
    //  
    FillChar(SHF, SizeOf(SHF), 0);
    //  ݒ
    with SHF do
    begin
        Wnd     := Application.Handle;
        wFunc   := FO_DELETE;
        pFrom   := PChar(FileNames);
        pTo     := nil;
        fFlags  := SHF_FLAG[ANeedConfirm];
        hNameMappings     := nil;
        lpszProgressTitle := nil;
    end;
    //  s
    if (SHFileOperation(SHF) <> 0) then
    begin
        raise Exception.Create(SysErrorMessage(GetLastError));
    end;
end;

(*
    =================================================================
    e|tH_̃pX擾

    http://www.wwlnk.com/boheme/delphi/vbtodel/daf0650.html
    =================================================================
*)
function    GetTempFolder(): string;
var
    Temp: array[0..MAX_PATH + 1] of Char;
begin
    GetTempPath(SizeOf(Temp), Temp);
    Result := Temp;
end;

(*
    =================================================================
    ċAItH_

    Subject: [Delphi-ML:4452] Re: fBNg\𓾂ɂ́H
    =================================================================
*)
function    FindFiles(const ADirPath: string; const ALevel: integer;
                ACallbackFile: TOnFindFile; ACallbackFolder: TOnFindFolder): Boolean;
var
    Path: string;
    Rec: TSearchRec;
    Skip, Cancel: Boolean;
begin

    Result := True;
    Path := ADirPath;

    // ȃtH_H
    if not DirectoryExists(Path) then Exit;

    // \ ĂH
    if Path[Length(Path)] <> '\' then Path := Path + '\';

    // Ȃ񂩓Ă邩ȁH
    if FindFirst(Path + '*.*', faAnyFile, Rec) = 0 then
    begin

        // Ă݂悤
        try
            repeat
                // tH_ƐetH_͖
                if (Rec.Name = '.') or (Rec.Name = '..') then Continue;

                // XLbvȂˁH
                Skip := False;

                // LZȂˁH
                Cancel := False;

                // tH_
                if (Rec.Attr and faDirectory) <> 0 then
                begin
                    ACallbackFolder(Path + Rec.Name, ALevel, Skip, Cancel);

                    // HXLbv...
                    if Skip then Continue;

                    // HLZ...
                    if Cancel then
                    begin
                        Result := False;
                        Break;
                    end;

                    // @@
                    Result := FindFiles(Path + Rec.Name, ALevel + 1, ACallbackFile, ACallbackFolder);

                    // BLZꂽB
                    if not Result then Break;
                end else
                begin

                    // R[obN
                    ACallbackFile(Path + Rec.Name, ALevel, Cancel);

                    // HLZ...
                    if Cancel then
                    begin
                        Result := False;
                        Break;
                    end;

                end;

            until
                // ǂ܂ł
                FindNext(Rec) <> 0;
        finally
            // Еt
            FindClose(Rec);
        end;

    end;
end;


(*
    =================================================================
    tH_I_CAO
    =================================================================
*)
function    SelectFolder(const ARoot: string = 'C:\'; ACaption: string = 'tH_̑I'): string;
var
    Dir: string;
begin
    // J
    if SelectDirectory(ACaption, ARoot, Dir) then
    begin
        Result := Dir;
        if Result[Length(Result)] <> '\' then Result := Result + '\';
    end else
    begin
        Result := '';
    end;
end;

(*
    ===========================================================================
    t@C̃vpeB_CAO\
    ---------------------------------------------------------------------------
    http://homepage1.nifty.com/MADIA/delphi/Win32API/Properties.htm
    ms-help://MS.PSDK.1033/shellcc/platform/shell/reference/structures/shellexecuteinfo.htm
    ===========================================================================
*)
procedure   OpenFilePropertyDialog(AFilePath: string);
var
    SH: TSHELLEXECUTEINFO;
    Mask: Cardinal;
begin
    //  
    FillChar(SH, SizeOf(SH), 0);
    Mask := SEE_MASK_NOCLOSEPROCESS or SEE_MASK_INVOKEIDLIST or SEE_MASK_FLAG_NO_UI;
    //  ݒ
    SH.cbSize       := Sizeof(SH);
    SH.fMask        := Mask;
    SH.Wnd          := Application.MainForm.Handle;
    SH.lpVerb       := 'properties';
    SH.lpParameters := nil;
    SH.lpDirectory  := nil;
    SH.lpIDList     := nil;
    SH.lpFile := PChar(AFilePath);
    //  s
    if not ShellExecuteEx(@SH) then
    begin
        raise Exception.Create(SysErrorMessage(GetLastError));
    end; 
end;

(*
    =====================================================================
    
    =====================================================================
*)

initialization

    AppExePath := Application.ExeName;
    AppExeName := ChangeFileExt(ExtractFileName(AppExePath), '');
    AppDirPath := ExtractFilePath(AppExePath);
    AppLogPath := ChangeFileExt(ExtractFileName(AppExePath), '.log');
    AppIniPath := ChangeFileExt(ExtractFileName(AppExePath), '.ini');
    AppDatPath := ChangeFileExt(ExtractFileName(AppExePath), '.dat');

end.
